//******************************************************************************
//*                                  uBee512                                   *
//*       An emulator for the Microbee Z80 ROM, FDD and HDD based models       *
//*                                                                            *
//*                                clock module                                *
//*                                                                            *
//*                   Copyright (C) 2007-2009 Stewart Kay                      *
//******************************************************************************
//
// Change CPU clock speed.
//
//==============================================================================
// ChangeLog (most recent entries are at top)
//==============================================================================
// v4.1.0 - 25 June 2009, S.J.Kay
// - Minor code change.
//
// v4.0.0 - 19 May 2009, S.J.Kay
// - Changes made to incorporate a Z80 API.  Specific Z80 code used here
//   has been replaced with code usable by any Z80 emulator.
// - Masked off appended register values (upper 8 bits) from port values where
//   these were required.
//
// v3.1.0 - 2 November 2008, S.J.Kay
// - Changed all printf() calls to use a local xprintf() function.
//
// v2.7.0 - 28 May 2008, S.J.Kay
// - Modified set_clock_speed() function call.
// - Added structure emu_t and moved variables into it.
//
// v2.3.0 - 6 January 2008, S.J.Kay
// - Added modio_t structure.
//
// v2.1.0 - 19 October 2007, S.J.Kay
// - Created a new file and implement the CPU clock speed change code.

//==============================================================================
/*
 *  uBee512 - An emulator for the Microbee Z80 ROM, FDD and HDD based models.
 *  Copyright (C) 2007-2009 Stewart Kay
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/ 
//==============================================================================

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "clock.h"
#include "z80api.h"
#include "z80.h"
#include "ubee512.h"
#include "function.h"

#include "macros.h"

//==============================================================================
// structures and variables
//==============================================================================
static int clock_changed;

extern emu_t emu;
extern model_t modelx;
extern modio_t modio;

//==============================================================================
// Clock Initialise.
//
//   pass: void
// return: int				0
//==============================================================================
int clock_init (void)
{
 return 0;
}

//==============================================================================
// Clock  de-initialise
//
//   pass: void
// return: int				0
//==============================================================================
int clock_deinit (void)
{
 return 0;
}

//==============================================================================
// Clock reset
//
// Set the clock speed back to normal
//
//   pass: void
// return: int				0
//==============================================================================
int clock_reset (void)
{
 if ((modelx.speed) && (clock_changed))
    {
     set_clock_speed(3.375, 0, 0);
     clock_changed = 0;
    } 
 return 0;
}

//==============================================================================
// Clock speed change - Port function
//
//   pass: uint16_t port
//         struct z80_port_read *port_s
// return: uint16_t			0xFFFF
//==============================================================================
uint16_t clock_r (uint16_t port, struct z80_port_read *port_s)
{
 int opcode;
 int clock;

 if (modelx.speed)
    {
     opcode = z80api_get_proc_opcode();

     if (opcode == 0xdb)
        clock = z80api_get_proc_reg_a();
     else
        clock = z80api_get_proc_reg_b() & B8(00000011);

     if (modio.clock)
        log_port("clock_r", "clock", port, clock);

     // only change the speed if it needs it.
     if ((emu.cpuclock == 3375000) && (clock == B8(00000010)))
        {
         set_clock_speed(6.750, 0, 0);
         clock_changed = 1;
        } 
     else
        {
         if ((emu.cpuclock == 6750000) && (clock == B8(00000000)))
            {
             set_clock_speed(3.375, 0, 0);
             clock_changed = 1;
            }
        }     
     return clock;
    }       
 return 0;
}
