/* function Header */

#ifndef HEADER_FUNCTION_H
#define HEADER_FUNCTION_H

#include <stdio.h>
#include <stdint.h>
#include <dirent.h>
#include "z80.h"

int time_get_secs (void);
uint64_t time_get_ms (void);
void time_delay_ms (int ms);
void time_wait_ms (int ms);
void toupper_string (char *dest, char *src);
int file_readline (FILE *fp, char *s, int size);
int copy_file (char *dest, char *src);
void convert_slash (char *path);
FILE *open_file (const char *path1, const char *path2, char *path3, const char *mode);
FILE *test_file (const char *path1, const char *path2, char *path3);
int string_search (char *strg_array[], char *strg_find);
int find_file_alias (char *filename, char *strg_search, char *strg_value);
void create_md5 (char *filename, char *md5);
#ifdef MINGW
char *strcasestr (char *haystack, char *needle);
int strcasecmp (const char *s1, const char *s2);
#endif
int xstrverscmp (const char *s1, const char *s2);

void function_emu_command (int cmd, int p);
void function_repeat_start (void);
void function_repeat_stop (void);
void function_update (void);

int16_t little_endian_16 (int16_t n);
uint16_t little_endian_u16 (uint16_t n);
int32_t little_endian_32 (int32_t n);
uint32_t little_endian_u32 (uint32_t n);
int64_t little_endian_64 (int64_t n);
uint64_t little_endian_u64 (uint64_t n);

int16_t big_endian_16 (int16_t n);
uint16_t big_endian_u16 (uint16_t n);
int32_t big_endian_32 (int32_t n);
uint32_t big_endian_u32 (uint32_t n);
int64_t big_endian_64 (int64_t n);
uint64_t big_endian_u64 (uint64_t n);

int function_init (void);
int function_deinit (void);
int function_reset (void);

uint16_t function_ubee_r (uint16_t port, struct z80_port_read *port_s);
void function_ubee_w (uint16_t port, uint8_t data, struct z80_port_write *port_s);

#define le16_to_host little_endian_16
#define host_to_le16 little_endian_16
#define leu16_to_host little_endian_u16
#define host_to_leu16 little_endian_u16          
#define le32_to_host little_endian_32
#define host_to_le32 little_endian_32
#define leu32_to_host little_endian_u32
#define host_to_leu32 little_endian_u32                                              
#define le64_to_host little_endian_64
#define host_to_le64 little_endian_64
#define leu64_to_host little_endian_u64
#define host_to_leu64 little_endian_u64

#define FILE_STR_SIZE      256
#define FILE_LIST_ENTRIES  128

#pragma pack(push, 1)  // push current alignment, alignment to 1 byte boundary

typedef union fp_t
   {
    uint64_t i;			        // forces 8 bytes of storage
    FILE *p;		                // store a 4 or 8 byte pointer
    DIR *d;
   }fp_t;

// version informnation response structure
typedef struct ub_version_t
   {
    uint16_t cmd;			// sub command
    uint16_t ver1;			// vers 1.x.x value
    uint16_t ver2;			// vers x.1.x value
    uint16_t ver3;			// vers x.x.1 value
   }ub_version_t;

// dump memory structure
typedef struct ub_dump_t
   {
    uint16_t cmd;			// sub command
    uint16_t id;			// 0xAA55 check ID    
    uint16_t addr;			// dump address
    uint16_t lines;			// lines
    uint16_t htype;			// header type
   }ub_dump_t;

// command and response structure
typedef struct ub_cmdres_t
   {
    uint16_t cmd;			// sub command
    uint16_t id;			// 0xAA55 check ID
    int16_t res;			// result
    fp_t fp;
   }ub_cmdres_t;

// file structure
typedef struct ub_file_t
   {
    uint16_t cmd;			// sub command
    uint16_t id;			// 0xAA55 check ID
    int16_t res;			// result
    fp_t fp;
    uint16_t addr1;			// address 1 in Z80 map
    uint16_t addr2;			// address 2 in Z80 map
    uint16_t addr3;			// address 3 in Z80 map
    uint16_t addr4;			// address 4 in Z80 map    
    int16_t size;			// size of object
    uint16_t num;			// number of chars
    int16_t val1;			// value 1
    int16_t val2;			// value 2    
   }ub_file_t;

// stdio input commands
typedef struct ub_stdio_input_t
   {
    uint16_t cmd;			// sub command
    int16_t res;			// result
   }ub_stdio_input_t;

// stdio character write commands
typedef struct ub_stdio_char_t
   {
    uint16_t cmd;			// sub command
    int16_t res;			// result
    uint16_t val;			// value
   }ub_stdio_char_t;

// stdio string commands
typedef struct ub_stdio_str_t
   {
    uint16_t cmd;			// sub command
    int16_t res;			// result
    uint16_t addr;			// address in Z80 map
   }ub_stdio_str_t;

typedef union ub_func_u
   {
    ub_version_t version;
    ub_dump_t dump;
    ub_stdio_input_t getchar;
    ub_stdio_char_t putchar;
    ub_stdio_str_t puts;
    ub_cmdres_t cmdres;
    ub_file_t file;
   }ub_func_u;

#pragma pack(pop)	// restore original alignment from stack

typedef struct func_t
   {
    int dump_addr;
    char file_list[FILE_STR_SIZE][FILE_LIST_ENTRIES];
    char file_run[FILE_STR_SIZE];
    char file_app[FILE_STR_SIZE];
    int file_load;
    int file_exec;
    int file_exit;
    int file_list_pos;
    int file_list_count;
   }func_t;

#endif     /* HEADER_FUNCTION_H */
