/* Sound Header */

#ifndef HEADER_SOUND_H
#define HEADER_SOUND_H

#include "z80.h"

void sound_volume (int vol);
int sound_init (void);
int sound_deinit (void);
int sound_reset (void);
void sound_update (void);
void sound_clock (int cpuclock, int method);
void sound_w (uint8_t data);
int sound_get_buffer_count (void);
void sound_command (int cmd);

#define SOUND_PROP 0
#define SOUND_3375 1

#define SOUND_VOLUME_PERCENT 15
#define SOUND_FREQUENCY 44100
#define SOUND_FORMAT AUDIO_U8
#define SOUND_CHANNELS 1
#define SOUND_BUFTYP uint8_t
#define SOUND_BUFSIZE_KB 500

// The lowest sound frequency has been increased to help re-syncing of audio
// when quiet, 20Hz now means re-syncs will occur after 50mS quiet periods.
#define SOUND_LOWESTFREQ 20.0

// The value used for SOUND_SAMPLES has been revised with version 2.7.0 as
// other changes have been made that can affect the audio quality. A very
// SLOW distorted sound is produced by some Windows installations if the
// value is not high enough.  For Unix this does not appear to be as
// critical.
//
// A value of 2048 for Windows allows a sample rate of up to 88200KHz to be
// used, a value of 1024 was found to be suitable for 44100KHz.

#ifdef MINGW
#define SOUND_SAMPLES 1024
#else
#define SOUND_SAMPLES 1024
#endif

// Sound hold-off delays and frequency speed adjustments. The frequency
// adjustment values affect the sound card rate only.  The audio should be a
// little slower than the Z80 emulation speed to avoid crackling and break
// up of the audio.

#ifdef MINGW
#define SOUND_HOLDOFF 50
#define SOUND_FREQADJUST -2.5
#else
#define SOUND_HOLDOFF 50
#define SOUND_FREQADJUST -2.5
#endif 

typedef struct sound_t
   {
    int enabled;
    int mute;
    int dobuffering;
    int holdoff;
    int volume;
    int samples;
    int frequency;
    float freq_adjust;
    int freq_lowest;
    int use_alg1;
   }sound_t;

#endif     /* HEADER_SOUND_H */
