/* VDU Header */

#ifndef HEADER_VDU_H
#define HEADER_VDU_H

#include "ubee512.h"
#include "z80.h"

#define VIDEORAMSIZE  2048
#define PCGRAMSIZE    2048

int vdu_init (void);
int vdu_deinit (void);
int vdu_reset (void);

void vdu_switchvideo_in (void);
void vdu_switchvideo_out (void);

uint8_t vdu_vidmem_r (uint32_t addr, struct z80_memory_read_byte *mem_s);
void vdu_vidmem_w (uint32_t addr, uint8_t data, struct z80_memory_write_byte *mem_s);

uint16_t vdu_colcont_r (uint16_t port, struct z80_port_read *port_s);
void vdu_colcont_w (uint16_t port, uint8_t data, struct z80_port_write *port_s);

uint16_t vdu_colwait_r (uint16_t port, struct z80_port_read *port_s);
void vdu_colwait_w (uint16_t port, uint8_t data, struct z80_port_write *port_s);

uint16_t vdu_lvdat_r (uint16_t port, struct z80_port_read *port_s);
void vdu_lvdat_w (uint16_t port, uint8_t data, struct z80_port_write *port_s);

void vdu_latchrom (uint16_t port, uint8_t data, struct z80_port_write *port_s);

typedef struct vdu_t
   {
// Alpha+ (Premium) variables, 8K for Screen, Colour and attribute, 32K for PCG
    char chr_rom[0x4000];
    char scr_ram[0x0800 * 4];
    char col_ram[0x0800 * 4];
    char att_ram[0x0800 * 4];
    char pcg_ram[0x0800 * 17];  // last bank is a dummy bank
   }vdu_t;

#endif /* HEADER_VDU_H */
