/* z80api Header */

#ifndef HEADER_Z80API_H
#define HEADER_Z80API_H

#include <stdint.h>

typedef struct z80regs_t
{
 int af;
 int bc;
 int de;
 int hl;
 int af_p;
 int bc_p;
 int de_p;
 int hl_p;
 int ix;
 int iy;
 int pc;
 int sp;
 int i;
 int r;
}z80regs_t;

// Action function type, for actions that can occur on Z80 state
// changes (e.g. Z80 halt)
typedef void (*z80api_action_fn_t)(void);
typedef enum { Z80_HALT = 0 } z80_event_t;
void z80api_register_action(z80_event_t when,
			    z80api_action_fn_t function);
void z80api_deregister_action(z80_event_t when,
			      z80api_action_fn_t function);

int z80api_init (void);
int z80api_deinit (void);
int z80api_reset (void);
int z80api_getpc (void);
void z80api_set_poll_tstates_def (int tstates);
void z80api_set_poll_tstates (int tstates, int repeats);
void z80api_execute (int tstates);
void z80api_execute_complete (void);
uint64_t z80api_get_tstates (void);
void z80api_nonmaskable_intr (void);
void z80api_maskable_intr (int vector);
int z80api_intr_possible (void);
void z80api_get_regs (z80regs_t *z80regs);
int z80api_get_proc_reg_a (void);
int z80api_get_proc_reg_b (void);
int z80api_get_proc_opcode (void);
void z80api_get_version (char *vers, int size);
void z80api_regdump (void);
uint8_t z80api_read_mem (int addr);
void z80api_write_mem (int addr, uint8_t value);

#endif /* HEADER_Z80API_H */
